@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(IO::Async::Notifier);
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Net::Async::AMQP - provides client interface to AMQP using L<IO::Async>
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -1055,12 +1055,14 @@ sub remove_pending {
 
 Writes data to the server.
 
+Returns a L<Future> which will resolve to an empty list when
+done.
+
 =cut
 
 sub write {
 	my $self = shift;
-	$self->stream->write(@_);
-	$self
+	$self->stream->write(@_)
 }
 
 =head2 process_frame
@@ -1219,7 +1221,8 @@ Takes the $frame instance followed by these optional named parameters:
 
 =back
 
-Returns $self.
+Returns a L<Future> which will resolve to an empty list
+when the frame has been written (this does not guarantee that the server has received it).
 
 =cut
 
@@ -1241,9 +1244,9 @@ sub send_frame {
 	my $data = $frame->to_raw_frame;
 
 #    warn "Sending data: " . Dumper($frame) . "\n";
-	$self->write($data);
-	$self->reset_heartbeat;
-	$self;
+	$self->write(
+		$data,
+	)->on_done($self->curry::reset_heartbeat)
 }
 
 =head2 header_bytes
