@@ -1,5 +1,5 @@
 package Net::Async::AMQP::Channel;
-$Net::Async::AMQP::Channel::VERSION = '0.027';
+$Net::Async::AMQP::Channel::VERSION = '0.028';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Net::Async::AMQP::Channel - represents a single channel in an MQ connection
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -48,6 +48,7 @@ request will fail.
 use Future;
 use curry::weak;
 use Class::ISA ();
+use Variable::Disposition qw(retain_future);
 use Data::Dumper;
 use Scalar::Util qw(weaken);
 
@@ -422,26 +423,31 @@ Called when the channel has been closed.
 sub on_close {
 	my ($self, $frame) = @_;
 
+	$self->{is_closed} = 1;
+
 	# ACK the close first - we have to send a close-ok
 	# before it's legal to reopen this channel ID
-	$self->send_frame(
-		Net::AMQP::Frame::Method->new(
-			method_frame => Net::AMQP::Protocol::Channel::CloseOk->new(
+	retain_future(
+		$self->send_frame(
+			Net::AMQP::Frame::Method->new(
+				method_frame => Net::AMQP::Protocol::Channel::CloseOk->new(
+				)
 			)
-		)
-	);
-
-	# It's important that the MQ instance knows
-	# about the channel closure first before we
-	# go ahead and dispatch events, since any
-	# subscribed handlers might go ahead and
-	# attempt to open the channel again immediately.
-	$self->amqp->channel_closed($self->id);
-	$self->bus->invoke_event(
-		'close',
-		code => $frame->reply_code,
-		message => $frame->reply_text,
-	);
+		)->then(sub {
+			# It's important that the MQ instance knows
+			# about the channel closure first before we
+			# go ahead and dispatch events, since any
+			# subscribed handlers might go ahead and
+			# attempt to open the channel again immediately.
+			$self->amqp->channel_closed($self->id);
+			$self->bus->invoke_event(
+				'close',
+				code => $frame->reply_code,
+				message => $frame->reply_text,
+			);
+			Future->done
+		})
+	)
 }
 
 =head2 send_frame
@@ -553,6 +559,7 @@ sub next_pending {
 	# First part of a frame. There's more to come, so stash a new future
 	# and return.
 	if($frame->isa('Net::AMQP::Frame::Header')) {
+		# Start with empty payload
 		$self->{incoming_message}{type} = $frame->header_frame->type;
 		if($frame->header_frame->headers) {
 			eval {
@@ -563,8 +570,9 @@ sub next_pending {
 		}
 
 		# Messages may be empty - in this case we'd have no body frames at all, we're done already:
-		unless($frame->body_size) {
-			$self->{incoming_message}{payload} = '';
+		if($frame->body_size) {
+			$self->{incoming_message}{pending} = $frame->body_size;
+		} else {
 			$self->bus->invoke_event(
 				message => @{$self->{incoming_message}}{qw(type payload ctag dtag rkey)},
 			);
@@ -575,14 +583,28 @@ sub next_pending {
 	}
 
 	# Body part of an incoming message.
-	# TODO should handle multiple chunks?
 	if($frame->isa('Net::AMQP::Frame::Body')) {
-		$self->{incoming_message}{payload} = $frame->payload;
-		$self->bus->invoke_event(
-			message => @{$self->{incoming_message}}{qw(type payload ctag dtag rkey)},
-		);
-		delete $self->{incoming_message};
-		return $self;
+		my $bytes = $frame->payload;
+		$self->{incoming_message}{payload} .= $bytes;
+		$self->{incoming_message}{pending} -= length $bytes;
+		if($self->{incoming_message}{pending} > 0) {
+			# We still have more to come, just return for now
+			return $self;
+		} elsif($self->{incoming_message}{pending} < 0) {
+			$self->close(
+				code => 500,
+				text => -$self->{incoming_message}{pending} . ' excess payload bytes detected in delivery'
+			);
+			delete $self->{incoming_message};
+			return $self;
+		} else {
+			# We have a full message now - hand it over to the event bus
+			$self->bus->invoke_event(
+				message => @{$self->{incoming_message}}{qw(type payload ctag dtag rkey)},
+			);
+			delete $self->{incoming_message};
+			return $self;
+		}
 	}
 
 	return $self unless $frame->can('method_frame') && (my $method_frame = $frame->method_frame);
@@ -609,6 +631,8 @@ sub next_pending {
 			ctag => $method_frame->consumer_tag,
 			dtag => $method_frame->delivery_tag,
 			rkey => $method_frame->routing_key,
+			payload => '',
+			payload_size => undef,
 		};
 		return $self;
 	}
@@ -661,9 +685,9 @@ sub bus { $_[0]->{bus} ||= Mixin::Event::Dispatch::Bus->new }
 
 =head2 future
 
-The underlying L<Future> for this channel which
-will resolve to the instance once the channel
-is open.
+The underlying L<Future> for this channel.
+
+Will resolve to the L<Net::Async::Channel> instance once the channel is open.
 
 =cut
 
@@ -687,6 +711,15 @@ sub as_string {
 	sprintf "Channel[%d]", $self->id;
 }
 
+=head2 closed
+
+Returns true if the channel has been closed, 1 if not (which could mean it is either not yet open,
+or that it is open and has not yet been closed by either side).
+
+=cut
+
+sub is_closed { shift->{is_closed} }
+
 =head2 closure_protection
 
 Helper method for marking any outstanding requests as failed when the channel closes.
